<?php

if (!defined('PLX_ROOT')) exit;

/**
 * Adaptation pour PluXml de la librairie Zoombox.
 * http://grafikart.github.io/Zoombox/
 * */
class zoomBox extends plxPlugin {

	const GALLERY_PATTERN = '@(<div\s+[^>]*data-gallery="([^"]*)">)[^<]*</div>@';
	# const FLICKR_GALLERY = '@<div\s[^>]*class="[^"]*\bflickr\b[^"]*"[^>]*>[^<]*</div>@';
	const FLICKR_PATTERN = '~(<div[^>]*(\s+data-flickr-\w+="[^"]*")+>)[^<]*</div>~'; # data-flickr-id may contains @

	public $fields = array(
		'theme'=>'string', 'opacity'=>'numeric', 'duration'=>'numeric', 'animation'=>'boolean',
		'width'=>'numeric', 'height'=>'numeric',
		'gallery'=>'boolean', 'autoplay'=>'boolean', 'overflow'=>'boolean',
		'fullscreen'=>'boolean', 'mobileGallery'=>'boolean',
		'flickr_size' => 'string', 'flickr_thumbnail' => 'string'
	);
	const DEFAULT_VALUES = array(
	    'theme'				=> 'zoombox',
	    'opacity'			=> 800,
	    'duration'			=> 1000,
	    'animation'			=> true,
	    'width'				=> 800,
	    'height'			=> 600,
	    'gallery'			=> true,
	    'autoplay'			=> false,
	    'overflow'			=> false,
	    'fullscreen'		=> true,
		'mobileGallery'		=> false,
	    'flickr_size'		=> 'b',
	    'flickr_thumbnail'	=> 'q'
	);

	const THEMES = 'zoombox lightbox prettyphoto darkprettyphoto simple golden';

	public  function __construct($default_lang) {

		# appel du constructeur de la classe plxPlugin (obligatoire)
		parent::__construct($default_lang);

		# droits pour accéder à la page config.php du plugin
		parent::setConfigProfil(PROFIL_ADMIN);

		$this->script_name = basename(strtolower($_SERVER['SCRIPT_NAME']),'.php');
		parent::addHook('plxMotorConstructLoadPlugins', 'plxMotorConstructLoadPlugins');
		# parent::addHook('AdminMediasFoot', 'AdminMediasFoot');
		parent::addHook('ThemeEndBody', 'ThemeEndBody');
		parent::addHook('plxMotorDemarrageEnd', 'plxMotorDemarrageEnd');
		parent::addHook('plxShowStaticContent', 'plxShowStaticContent');
		parent::addHook('plxFeedDemarrageEnd', 'plxFeedDemarrageEnd');
	}

	/*
	public function onActivate() {
		foreach($this->params as $field=>$type) {
			if (array_key_exists($field, $this->params_default)) {
				$this->setParam($field, $this->params_default[$field], ($type != 'boolean') ? $type : 'numeric');
			}
		}
		$this->saveParams();
	}
	 * */

	public function printConfigJS($id) {
?>
<script type="text/javascript">
	(function() {
		'use strict';
		const btn = document.getElementById('<?= $id; ?>');
		if(btn != null) {
			btn.onclick = function(event) {
				const params = JSON.parse('<?= json_encode(SELF::DEFAULT_VALUES) ?>');
				for(var key in params) {
					const entry = document.getElementById('id_' + key);
					if(entry == null) {
						console.error('#id_' + key + ' element not found');
						continue;
					}
					if(entry.type == 'checkbox') {
						entry.checked = params[key];
					} else {
						entry.value = params[key];
					}
				}
				event.preventDefault();
			}
		}
	})();
</script>
<?php
	}

	private function _get_plumxl_gallery() {
		global $plxShow, $output;

		// crée une galerie pour les images stockées par Pluxml
		if (preg_match('#(<(?:div|p)\s+[^>]*id="pl_gallery"[^>]*>)[^<]*</div>#', $output, $matches)) {
			if (preg_match('#data-src="([^"]+)"#', $matches[1], $match_src)) {
				$folder = $match_src[1];
				if (substr($folder, -1) != '/')
					$folder .= '/';
				$root = substr(dirname(__FILE__), 0, -strlen($plxShow->plxMotor->aConf['racine_plugins'].__CLASS__));
				$filesList = glob($root.$plxShow->plxMotor->aConf['medias'].$folder.'*.tb.{jpg,jpeg,png,gif}', GLOB_BRACE);
				$innerHTML = PHP_EOL;
				foreach ($filesList as $filename) {
					$src = substr($filename, strlen($root));
					$sizes = getimagesize($filename);
					$title = ucfirst(substr(pathinfo($filename, PATHINFO_FILENAME), 0, -3));
					$innerHTML .= <<< INNER_HTML
						<div><img src="$src" alt="image" {$sizes[3]} title="$title" /><p>$title</p></div>

INNER_HTML;
				}
				$output = str_replace($matches[0], $matches[1].plxUtils::rel2abs($plxShow->plxMotor->racine, $innerHTML).'</div>', $output);
			}
		}
	}

	public function build_gallery($folder, $root) {
		$folder =  trim($folder, '/');
		$files = glob("$root$folder/*.tb.{jpg,jpeg,gif,png}", GLOB_BRACE);
		$output = array();
		$offset = strlen(PLX_ROOT);
		foreach($files as $f) {
			$src = substr($f, $offset);
			$title= ucfirst(str_replace('_', ' ', preg_replace('@.*/(.*)\.tb\.(?:gif|png|jpe?g)$@', '$1', $f)));
			$imagesize = getimagesize($f);
			$output[] = <<< IMG
<img src="$src" title="$title" {$imagesize[3]} />
IMG;
		}
		return implode("\n", $output);
	}

	public function get_flickr($capture) {
		if(preg_match_all('@data-flickr-(\w+)="([^"]+)"@', $capture, $matches, PREG_SET_ORDER)) {
			$params = array();
			foreach($matches as $infos) {
				$params[$infos[1]] = $infos[2];
			}
			$service = 'photos_public';
			$id = (!empty($infos['id']) and preg_match('~^[\w@]{12,}$~', $infos['id'])) ? true : false;
			if(array_key_exists('service', $params)) {
				$serv = $params['service'];
				if(
					in_array($serv, explode(' ',  'friends faves groups')) and
					!empty($id)
				) {
					$service = ($serv == 'groups') ? 'groups_pool' : "photos_$serv";
				}
				unset($params['service']);
			}
			# https://www.flickr.com/services/api/response.php.html
			$params['format'] = 'php_serial';
			$url = "https://api.flickr.com/services/feeds/{$service}.gne/?";
			$query = array_map(
				function($a, $b) {
					return urlencode($a).'='.urlencode($b);
				},
				array_keys($params),
				array_values($params)
			);
			$url .= implode('&', $query);
			$ch = curl_init();
			curl_setopt_array($ch, array(
				CURLOPT_URL				=> $url,
				CURLOPT_FOLLOWLOCATION	=> true,
				CURLOPT_RETURNTRANSFER	=> true
			));
			$resp = curl_exec($ch);
			curl_close($ch);
			if($resp) {
				$catalogue = unserialize($resp);
				$figs = array();
				foreach($catalogue['items'] as $item) {
					$figs[] = <<< FIGURE
<figure>
	<figcaption>{$item['title']}</figcaption>
	<p><a href="{$item['']}"><img src="{$item['thumb_url']}"></a></p>
	<div>
	{$item['description']}
	</div>
</figure>
FIGURE;
				}
				$album = implode("\n", $figs);
				$infos = print_r($catalogue, true);
				return <<< HEADER
<!-- FlickR ====================================
$url
$infos
-->
<div>
	<img src="{$catalogue['image']}" />
	<a href="{$catalogue['url']}">{$catalogue['title']}</a>
</div>
<div>
$album
</div>\n
HEADER;
			} else {
				return 'Not found';
			}

		}
	}

/* ================== Hooks ================== */

	/**
	 * Détermine l'URL absolue du dossier de ce plugin.
	 * */
	public function plxMotorConstructLoadPlugins() {
		$code = <<< 'code'
<?php
	$this->plxPlugins->aPlugins['##CLASS##']->racine = $this->urlRewrite($this->aConf['racine_plugins'].'##CLASS##/');
?>
code;
		echo str_replace('##CLASS##', __CLASS__, $code);
	}

	public function plxShowStaticContent() {
		$code = <<< 'CODE'
<?php
$replaces = array();
if(preg_match_all('##PATTERN##', $output, $matches, PREG_SET_ORDER)) {
	foreach($matches as $match) {
		$gallery = $this->plxMotor->plxPlugins->aPlugins['##CLASS##']->build_gallery(
			$match[2],
			PLX_ROOT.$this->plxMotor->aConf['medias']
		);
		$replaces['@'.$match[0].'@'] = "{$match[1]}$gallery</div>";
	}
}
/* -------- Flick'R --------- */
if(function_exists('curl_init') and preg_match_all('##FLICKR##', $output, $matches, PREG_SET_ORDER)) {
	foreach($matches as $match) {
		$gallery = $this->plxMotor->plxPlugins->aPlugins['##CLASS##']->get_flickr($match[1]);
		if(!empty($gallery)) {
			$replaces['~'.$match[0].'~'] = "{$match[1]}$gallery</div>";
		}
	}
}
if(!empty($replaces)) {
	$output = preg_replace(array_keys($replaces), array_values($replaces), $output);
}
?>
CODE;
		$replaces = array(
			'##PATTERN##'	=> self::GALLERY_PATTERN,
			'##FLICKR##'	=> self::FLICKR_PATTERN,
			'##CLASS##'		=> __CLASS__
		);
		echo str_replace(array_keys($replaces), array_values($replaces), $code);
	}

	public function ThemeEndBody($params) {
		$buf = glob(__DIR__ . '/jquery*.js');
		if(!empty($buf)) {
			$files = array_map(function($item) { return basename($item); }, $buf);
			if(count($buf) > 1) {
				usort($files, function($a, $b) { return version_compare($b, $a); }); // reverse order !
			}
		}
?>
	<script
		type="text/javascript"
		src="<?= $this->racine ?>zoomBox/zoombox.js"
		data-jquery="<?= $files[0] ?>"
		data-zoombox='<?= parent::getParam('settings') ?>'
	></script>
<?php
	}

	/**
	 * Construit une galerie d'images pour les balises <div data-gallery="...">.
	 * */
	public function plxMotorDemarrageEnd() {
		$code = <<< 'CODE'
<?php
$pattern = '@(<div\s+[^>]*data-gallery="([^"]*)">)[^<]*</div>@';
switch($this->mode) {
	case 'static':
		break;
	case 'article':
		break;
	case 'home':
		if(!empty($this->plxRecord_arts)) {
			$result = $this->plxRecord_arts->result;
			for($i=0, $iMax=count($result); $i < $iMax; $i++) {
				if(preg_match($pattern, $result[$i]['content'], $matches)) {
					$gallery = $this->plxPlugins->aPlugins['##CLASS##']->build_gallery(
						$matches[2],
						PLX_ROOT.$this->aConf['medias']
					);
					$result[$i]['content'] = preg_replace(
						$pattern,
						"\1{$gallery}</div>",
						$result[$i]['content']
					);
				}
			}
		}
		break;
}
?>
CODE;
		echo str_replace('##CLASS##', __CLASS__, $code);
	}

	/**
	 * Dans un flux RSS, insère les balises <img .. /> avec src="mon-image.tb.(?:jpe?g|png|gif)" dans une balise <a>
	 * avec href pointant sur l'image en taille réelle.
	 * */
	public function plxFeedDemarrageEnd() {
		$code = <<< 'CODE'
<?php
if($this->mode == 'article') {
	$replaces = array(
		'@<img\s+.*src="([^"]*)\.tb(\.(?:jpe?g|gif|png)"[^>]*)>@' => '<a href="\1\2">\0</a>'
	);
	$output = ob_get_clean();
	echo preg_replace(array_keys($replaces), array_values($replaces), $output);
}
?>
CODE;
	echo $code;
	}

}
